/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core;

import java.io.File;

import org.apache.log4j.Logger;

import fr.hypik.developpez.defis.soma.core.engine.impl.SimpleSomaEngine;
import fr.hypik.developpez.defis.soma.core.exception.CsvMalformeException;
import fr.hypik.developpez.defis.soma.core.factory.CsvFactory;
import fr.hypik.developpez.defis.soma.core.factory.PieceFactory;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * Permet de tester unitairement la rsolution d'un cube de soma en ligne de commande.
 * 
 * @author Pierrick HYMBERT
 *
 */
public class SomaMain {

	/**
	 * Le logger de la classe
	 */
	private static final Logger logger = Logger.getLogger(SomaMain.class);
	
	/**
	 * @param args Les arguments de la ligne de commande
	 */
	public static void main(String[] args) {
		String puzzleFileName = null;
		boolean allSolutions = false;
		boolean printHelp = false;
		
		//check les arguments
		for(String arg : args)
		{
			if( arg.startsWith("-") )
			{
				if( arg.equalsIgnoreCase("-a") || arg.equalsIgnoreCase("--all"))
				{
					allSolutions = true;
				}
				else if( arg.equalsIgnoreCase("-h") || arg.equalsIgnoreCase("--help"))
				{
					printHelp = true;
				}
				else
				{
					logger.error("Paramtre non reconnu: " + arg);
					printHelp = true;
				}
			}
			else
				puzzleFileName = arg;
		}
		
		// Affiche l'aide si les arguments l'implique
		if( puzzleFileName == null || printHelp )
		{
			printHelp();
		}
		else
		{
			try
			{
				// Charge les pices de soma
				try
				{
					SimpleSomaEngine.get().setSomaPieces(PieceFactory.get().getAllSomaPieces());
				}catch (CsvMalformeException e) {
					throw new IllegalAccessException("Les pieces du cube de soma sont erronees: " + e);
				}
				
				// Lit le fichier
				File puzzleFile = new File(puzzleFileName);
				Piece puzzle = new Piece(puzzleFile.getName(), CsvFactory.get().readCubes(puzzleFile));
				
				// Recherche de solution
				if( allSolutions )
				{
					SimpleSomaEngine.get().getAllSolutions(puzzle.getCubes());
				}
				else
				{
					SimpleSomaEngine.get().getSolution(puzzle.getCubes());
				}
				
			}catch (CsvMalformeException e) {
				logger.error("Le fichier CSV est mal forme : " + e, e);
			}catch (Exception e) {
				logger.error("Une erreur inattendue s'est produite : " + e, e);
			}
			finally
			{
				logger.info("Merci d'avoir utilise ce programme :-)");
			}
		}
	}
	
	/**
	 * Affiche l'aide
	 */
	private static void printHelp()
	{
		logger.info("soma-cube-1.0 : Resolution d'un cube de SOMA");
		logger.info("@author Pierrick HYMBERT");
		logger.info("@see Defi developpez.com SOMA Cube : http://c.developpez.com/defis/5-Cube-Soma/");
		logger.info("java -cp ... fr.hypik.developpez.defis.soma.core.SomaMain [-ah] puzzleFile.csv");
		logger.info("Options :");
		logger.info("         -a --all  : Trouve toutes les solutions de la figure");
		logger.info("         -h --help : Affiche cette aide");
	}
}
