/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core.engine;

import java.util.List;

import fr.hypik.developpez.defis.soma.core.exception.PuzzleInvalideException;
import fr.hypik.developpez.defis.soma.core.listener.ISomaEngineProgressListener;
import fr.hypik.developpez.defis.soma.core.model.Cube;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * Description de comportement d'un moteur de resolution de cube soma
 * 
 * @author Pierrick HYMBERT
 *
 */
public interface ISomaEngine {

	/**
	 * La borne minimale du jeu
	 */
	public static final int BORNE_MIN = 0;

	/**
	 * La borne maximale du jeu
	 */
	public static final int BORNE_MAX = 17;
	
	/**
	 * Donne la solution du cube
	 * @param soma Le cube de soma  raliser
	 * @return Les pices de soma correctement assemble
	 * @throws PuzzleInvalideException Si le puzzle est invalide
	 */
	public Piece[] getSolution(Cube[] soma) throws PuzzleInvalideException;
	
	/**
	 * Donne la liste de toutes les solutions du cube
	 * @param soma Le cube de soma  raliser
	 * @return Les pices de soma correctement assemble
	 * @throws PuzzleInvalideException Si le puzzle est invalide
	 */
	public List<Piece[]> getAllSolutions(Cube[] soma) throws PuzzleInvalideException;
	
	/**
	 * Les septs pices de soma
	 * @param somaPieces
	 */
	public void setSomaPieces(Piece[] somaPieces);
	
	/**
	 * Ajoute un listener de recherche de solution
	 * @param somaEngineProgressListener Le listener
	 */
	public void addSomaEngineProgressListener(ISomaEngineProgressListener somaEngineProgressListener);
	
	/**
	 * Supprime un listener de recherche de solution
	 * @param somaEngineProgressListener Le listener
	 */
	public void removeSomaEngineProgressListener(ISomaEngineProgressListener somaEngineProgressListener);
	
	/**
	 * Interrompt la recherche de solution
	 */
	public void interrupt();
}
