/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core.engine.impl;

import java.util.ArrayList;
import java.util.Collection;

import fr.hypik.developpez.defis.soma.core.engine.ISomaEngine;
import fr.hypik.developpez.defis.soma.core.listener.ISomaEngineProgressListener;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * La classe de base des moteurs de rsolutions du cube de soma
 * @author Pierrick HYMBERT
 */
public abstract class AbstractSomaEngine implements ISomaEngine {
	/**
	 * Les listeners
	 */
	protected Collection<ISomaEngineProgressListener> listeners = new ArrayList<ISomaEngineProgressListener>();
	/**
	 * Toutes les pices de SOMA
	 */
	protected Piece[] somaPieces;
	
	/**
	 * 
	 */
	private boolean interrupted;

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.hypik.developpez.defis.soma.core.engine.ISomaEngine#
	 * addSomaEngineProgressListener
	 * (fr.hypik.developpez.defis.soma.core.listener
	 * .ISomaEngineProgressListener)
	 */
	public void addSomaEngineProgressListener(
			ISomaEngineProgressListener somaEngineProgressListener) {
		this.listeners.add(somaEngineProgressListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.hypik.developpez.defis.soma.core.engine.ISomaEngine#
	 * removeSomaEngineProgressListener
	 * (fr.hypik.developpez.defis.soma.core.listener
	 * .ISomaEngineProgressListener)
	 */
	public void removeSomaEngineProgressListener(
			ISomaEngineProgressListener somaEngineProgressListener) {
		this.listeners.remove(somaEngineProgressListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * fr.hypik.developpez.defis.soma.core.core.engine.ISomaEngine#setSomaPieces
	 * (fr.hypik .developpez.defis.soma.model.Piece[])
	 */
	public void setSomaPieces(Piece[] somaPieces) {
		this.somaPieces = somaPieces;
	}
	
	/**
	 * Interrompt la recherche de solution
	 */
	public void interrupt()
	{
		this.interrupted = true;
	}
	
	/**
	 * Dmarre la recherche de solution
	 */
	protected void start()
	{
		this.interrupted = false;
	}
	
	/**
	 * @return True si la recherche est arrete, faux sinon
	 */
	public boolean isInterrupted() {
		return interrupted;
	}
}
