/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core.engine.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import fr.hypik.developpez.defis.soma.core.exception.PuzzleInvalideException;
import fr.hypik.developpez.defis.soma.core.listener.ISomaEngineProgressListener;
import fr.hypik.developpez.defis.soma.core.model.Axe;
import fr.hypik.developpez.defis.soma.core.model.Cube;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * Implmentation du moteur de resolution suivant le premier algorithme dcrit
 * ici: http://web.me.com/rouxjeanbernard/Site/AM/html/amch94.html
 * 
 * Par rapport au moteur simple, cet algorithme ne calcule pas toutes les combinaisons de toutes les pices ds le dbut, mais de faon incrmentale
 * 
 * @author Pierrick HYMBERT.
 * 
 */
public class EnhancedSimpleSomaEngine extends SimpleSomaEngine {

	/**
	 * Le logger de la classe
	 */
	private static final Logger logger = Logger
			.getLogger(EnhancedSimpleSomaEngine.class);
	/**
	 * L'instance de CSV factory
	 */
	private static final EnhancedSimpleSomaEngine singleton = new EnhancedSimpleSomaEngine();
	
	/**
	 * @return L'instance de CSV factory
	 */
	public static EnhancedSimpleSomaEngine get()
	{
		return singleton;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * fr.hypik.developpez.defis.soma.core.core.engine.ISomaEngine#getAllSolutions(fr.
	 * hypik.developpez.defis.soma.model.Cube[])
	 */
	public List<Piece[]> getAllSolutions(Cube[] soma) throws PuzzleInvalideException {
		// Le puzzle  atteindre
		Piece solution = new Piece("Soma", soma);
		
		logger.info("Recherche toutes les solutions pour le puzzle: " + solution);
		
		// Toutes les positions de toutes les pices dans la solution
		Map<Integer, List<Piece>> allSolutionsByPiece = new HashMap<Integer, List<Piece>>();
		
		// Le nombre de solutions testees
		long totalSolutionTestees = 0;
		
		// Le nombre de solutions testees
		long solutionTestees = 0;
		
		// La liste de toutes les solutions
		List<Piece[]> allSolutions = new ArrayList<Piece[]>();
		
		// La solution courante
		Map<Integer, Integer> solutionPiecesCourante = new HashMap<Integer, Integer>();
		
		// Les solutions dj utilisees
		List<Map<Integer, Integer>> solutionInterdites = new ArrayList<Map<Integer, Integer>>();
		
		// L'avancement de la recherche incrmentale de solutions par pice
		Map<Integer, PieceSolutionProgress> pieceSolutionProgressByPiece = new HashMap<Integer, PieceSolutionProgress>();

		// Recherche toutes les solutions
		try
		{
			while( ( solutionTestees = getSolution(solution, pieceSolutionProgressByPiece, allSolutionsByPiece, solutionPiecesCourante, solutionInterdites) ) > 0 )
			{
				// La nouvelle map des solutions
				Map<Integer, Integer> solutionPiecesCouranteBis = new HashMap<Integer, Integer>();
				
				// Incrmente le nombre total de solution testees
				totalSolutionTestees += solutionTestees;
				
				// Construit la solution
				Piece[] finalSolution = new Piece[somaPieces.length];
				for(int i = 0; i < somaPieces.length; i++)
				{
					Piece piece = allSolutionsByPiece.get(i)
					.get(solutionPiecesCourante.get(i));
					finalSolution[i] = piece;
					solutionPiecesCouranteBis.put(i, solutionPiecesCourante.get(i));
					logger.info(piece);
					if( logger.isDebugEnabled() )
					{
						logger.debug("Piece " + piece.getNom() + " solution " + solutionPiecesCourante.get(i) + "/" + allSolutionsByPiece.get(i).size());
					}
				}
				
				// Incrmente la dernire combinaison
				solutionPiecesCourante.put(somaPieces.length-1, solutionPiecesCourante.get(somaPieces.length-1) + 1);
				
				// Ajoute la solution  la liste des solutions
				allSolutions.add(finalSolution);
				
				// Ajoute la solution courante  la liste des solutions
				solutionInterdites.add(solutionPiecesCouranteBis);
				
				logger.info("Nombre de solutions trouves: " + allSolutions.size());

				// Previent les listeners
				for(ISomaEngineProgressListener listener :  listeners)
					listener.solutionTrouvee(finalSolution, solution);
			}
		}
		catch (PuzzleInvalideException e) {
			// Lve l'exception si on a trouve aucune solution
			if( allSolutions.size() == 0)
				throw e;
		}
		
		// Log le nombre de solution trouves
		logger.info("Nombre de solutions trouves: " + allSolutions.size() + ", nombre de combinaisons testes: " + totalSolutionTestees);

		return allSolutions;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * fr.hypik.developpez.defis.soma.core.core.engine.ISomaEngine#getSolution(fr.hypik
	 * .developpez.defis.soma.model.Cube[])
	 */
	public Piece[] getSolution(Cube[] soma) throws PuzzleInvalideException {
		Piece solution = new Piece("Soma", soma);
		
		logger.info("Recherche une solution pour le puzzle: " + solution);
		
		// Toutes les positions de toutes les pices dans la solution
		Map<Integer, List<Piece>> allSolutionsByPiece = new HashMap<Integer, List<Piece>>();
		
		// La solution courante
		Map<Integer, Integer> solutionPiecesCourante = new HashMap<Integer, Integer>();
		
		// L'avancement de la recherche incrmentale de solutions par pice
		Map<Integer, PieceSolutionProgress> pieceSolutionProgressByPiece = new HashMap<Integer, PieceSolutionProgress>();

		// Recherche une solution
		Piece[] finalSolution = null;
		
		getSolution(solution, pieceSolutionProgressByPiece, allSolutionsByPiece, solutionPiecesCourante, null);
		
		// Construit la solution
		finalSolution = new Piece[somaPieces.length];
		for(int i = 0; i < somaPieces.length; i++)
		{
			Piece piece = allSolutionsByPiece.get(i)
			.get(solutionPiecesCourante.get(i));
			finalSolution[i] = piece;
			logger.info(piece);
			if( logger.isDebugEnabled() )
			{
				logger.debug("Piece " + piece.getNom() + " solution " + solutionPiecesCourante.get(i) + "/" + allSolutionsByPiece.get(i).size());
			}
		}
		
		// Previent les listeners
		for(ISomaEngineProgressListener listener :  listeners)
			listener.solutionTrouvee(finalSolution, solution);
		
		return finalSolution;
	}

	/**
	 * Recherche une solution
	 * 
	 * @param solution Le puzzle a atteindre
	 * @param pieceSolutionProgressByPiece L'avancement de la recherche incrmentale de solution
	 * @param allSolutionsByPiece Toutes les positions de toutes les pices dans la solution
	 * @param solutionPiecesCourante
	 *            La solution courante. La cl est l'index de la pice . La
	 *            valeur est l'index de la solution pour cette pice dans la
	 *            liste de toutes les solution de la piece
	 * @param solutionInterdites La liste des combinaisons interdites
	 * @return Le nombre de solutions testes
	 * @throws PuzzleInvalideException Si le puzzle est irralisable
	 */
	protected long getSolution(Piece solution, Map<Integer, PieceSolutionProgress> pieceSolutionProgressByPiece,
			Map<Integer, List<Piece>> allSolutionsByPiece,
			Map<Integer, Integer> solutionPiecesCourante
			,List<Map<Integer, Integer>> solutionInterdites) throws PuzzleInvalideException {
		
		// Le nombre de solutions testes
		long nbSolutionsTestees = 0;
		
		// Le nom de la pice courante  placer
		int pieceCourante = Math.max(0, solutionPiecesCourante.size() - 1);

		// Definie la premire solution pour la premire pice
		if( !solutionPiecesCourante.containsKey(pieceCourante) )
			solutionPiecesCourante.put(pieceCourante, 0);
		
		// Fixe si la dernire solution tait la dernire combinaison de la dernire pice
		while( solutionPiecesCourante.containsKey(pieceCourante)
				&& allSolutionsByPiece.containsKey(pieceCourante)
				&& solutionPiecesCourante.get(pieceCourante) == allSolutionsByPiece.get(pieceCourante).size())
		{
			pieceCourante--;
		}
		
		// Log l'entre dans la mthode de recherche de solution
		logger.info("Recherche d'une solution pour le puzzle " + solution);
		
		// Le flag indiquant que l'on a trouv une solution
		boolean solutionTrouvee = false;

		// Tant que l'on n'a pas test toutes les solutions ou pas trouv de
		// solution
		// CAD:
		// Tant que
		// L'on a pas cherch de solution pour la premire pice
		// l'indice de la solution de la premire pice n'est pas suprieur au nombre de solution de la premire pice
		while ( !allSolutionsByPiece.containsKey(0)
				||
				solutionPiecesCourante
						.get(0) < allSolutionsByPiece.get(0).size() ) {
			try
			{
				// L'indice de la solution de la pice courante
				int solutionPieceCourante = solutionPiecesCourante.get(pieceCourante);
				
				// Vrifie si l'on a un avancement
				if( !pieceSolutionProgressByPiece.containsKey(pieceCourante) )
				{
					pieceSolutionProgressByPiece.put(pieceCourante, new PieceSolutionProgress());
					allSolutionsByPiece.put(pieceCourante, new ArrayList<Piece>());
				}
				
				// Toutes les solutions de la pice courante
				List<Piece> allSolutionsPiece = allSolutionsByPiece.get(pieceCourante);
				
				// L'avancement de la recherche de solution pour la pice courante
				PieceSolutionProgress pieceSolutionProgress = pieceSolutionProgressByPiece.get(pieceCourante);
				
				// Incrmente la recherche de solution pour cette pice
				buildPieceSolutions(somaPieces[pieceCourante], solution, allSolutionsPiece, pieceSolutionProgress);
				
				// La solution de la pice courante
				Piece piece = allSolutionsPiece.get(solutionPieceCourante);
				
				// Log l'avancement
				if( logger.isDebugEnabled() )
				{
					logger.debug("Recherche une solution pour la pice " + piece.getNom() + "(" + pieceCourante + "/" + somaPieces.length + ") " + solutionPieceCourante + "/" + allSolutionsPiece.size() + ", recherche termines pour cette piece " + pieceSolutionProgress.rechercheTerminee() + ", solutions testes " + nbSolutionsTestees );
				}
				
				// Incrmente le nombre de solutions testes
				nbSolutionsTestees++;
				
				// Check si toutes les pices sont incluses
				// Ou si la solution existe dej
				if( collisionExiste(allSolutionsByPiece, solutionPiecesCourante, pieceCourante) 
						|| (  pieceCourante + 1 == somaPieces.length && solutionInterdites != null && solutionInterdites.contains(solutionPiecesCourante)))
				{
					// Incrmente la recherche de solution pour cette pice
					buildPieceSolutions(somaPieces[pieceCourante], solution, allSolutionsPiece, pieceSolutionProgress);
					
					// Le puzzle n'est pas terminee
					// Soit on passe  la solution suivante soit on recule
					if( logger.isDebugEnabled() )
					{
						logger.debug("Cette solution est invalide pour la pice " + piece.getNom() + "(" + pieceCourante + "/" + somaPieces.length + ") " + solutionPieceCourante + "/" + allSolutionsPiece.size() + ", recherche termines pour cette piece " + pieceSolutionProgress.rechercheTerminee() + ", solutions testes " + nbSolutionsTestees );
					}
					if( ++solutionPieceCourante == allSolutionsPiece.size() && pieceCourante > 0 && pieceSolutionProgress.rechercheTerminee())
					{
						do
						{
							// Log le recul
							if( logger.isDebugEnabled() )
							{
								logger.debug("Toutes les solutions pour la pice " + piece.getNom() + "(" + ( pieceCourante - 1 )+ "/" + somaPieces.length + ") ont t teste, retour  la pice prcdente");
							}
							
							// On reinitialize la solution de la pice suivante  zero
							solutionPieceCourante = 0;
							
							// Set la nouvelle position de cette pice dans les solutions  tester
							solutionPiecesCourante.put(pieceCourante, solutionPieceCourante);
							
							// On retourne  la pice prcdente
							pieceCourante--;
							
							// Incrmente les solutions de la pice prcdente
							solutionPieceCourante = solutionPiecesCourante.get(pieceCourante) + 1;
							
							//
							pieceSolutionProgress = pieceSolutionProgressByPiece.get(pieceCourante);
							
							// Toutes les solutions de la pice courante
							allSolutionsPiece = allSolutionsByPiece.get(pieceCourante);
							
							buildPieceSolutions(somaPieces[pieceCourante], solution, allSolutionsPiece, pieceSolutionProgress);
							
						}while( solutionPieceCourante == allSolutionsByPiece.get(pieceCourante).size() && pieceCourante > 0 && pieceSolutionProgress.rechercheTerminee());
					}
					// Set la nouvelle position de cette pice dans les solutions  tester
					solutionPiecesCourante.put(pieceCourante, solutionPieceCourante);
				}
				else
				{
					// Vrifie si on a termin le puzzle
					if( pieceCourante + 1 == somaPieces.length)
					{
						solutionTrouvee = true;
						break;
					}
					else
					{
						if( logger.isDebugEnabled() )
						{
							logger.debug("Passage  la pice suivante.");
						}
						
						// On passe  la pice suivante
						pieceCourante ++;						
						
						// Initialise la pice courante
						solutionPiecesCourante.put(pieceCourante, 0);
					}
				}
			}
			catch (Exception e) {
				e.printStackTrace();
				logger.error(e, e);
			}
		}
		
		// Log la fin de l'algo
		logger.info("Puzzle test: " + solution);
		logger.info("Algorithme de recherche simple terminee, solution trouvee ? " + solutionTrouvee);
		logger.info("Combinaisons testes: " + nbSolutionsTestees);
		
		// Lve l'exception solution non trouvee
		if( !solutionTrouvee )
		{
			throw new PuzzleInvalideException();
		}
		
		// Retourne le nombre de solution testees
		return nbSolutionsTestees;
	}

	/**
	 * Vrifie si une collision existe dans la solution actuelle Si aucune pice
	 * est en collision, alors le puzzle est termine
	 * 
	 * @param allSolutionsByPiece
	 *            Toutes les positions de toutes les pices dans la solution
	 * @param solutionPiecesCourante
	 *            La solution courante
	 * @param pieceCourante
	 *            L'indice de la pice courante
	 * @return
	 */
	protected boolean collisionExiste(
			Map<Integer, List<Piece>> allSolutionsByPiece,
			Map<Integer, Integer> solutionPiecesCourante, int pieceCourante) {

		Piece nouvellePiece = allSolutionsByPiece.get(pieceCourante).get(
				solutionPiecesCourante.get(pieceCourante));
		
		for (int i = 0; i < pieceCourante; i++) {
			// La pice a tester
			Piece pieceI = allSolutionsByPiece.get(i).get(
					solutionPiecesCourante.get(i));
			
			if (pieceI.collide(nouvellePiece)) {
				return true;
			}
		}

		return false;
	}
	
	/**
	 * Recherche incrmentale de toutes les solution de la piece
	 * S'arrete lorsqu'une solution a t trouve
	 * 
	 * @param piece
	 *            La pice  placer
	 * @param solution
	 *            La solution
	 * @param allPieceSolutions
	 *            Toutes les solutions pour la pice
	 * @param pieceSolutionProgress
	 *            L'avancement de la recherche incrmentale
	 * @throws PuzzleInvalideException Si le puzzle est invalide
	 */
	protected void buildPieceSolutions(Piece piece, Piece solution, List<Piece> allPieceSolutions, PieceSolutionProgress pieceSolutionProgress) throws PuzzleInvalideException{
		if( pieceSolutionProgress.rechercheTerminee() )
			return;
		
		if (logger.isDebugEnabled()) {
			logger.debug("Recherche des solutions pour la piece "
					+ piece.getNom());
		}
		
		// calcul la distance max sur les axes de la solution
		int distanceMaxX = solution.distanceMax(Piece.ORIGINE, Axe.Abscisse) + 1;
		int distanceMaxY = solution.distanceMax(Piece.ORIGINE, Axe.Ordonnee) + 1;
		int distanceMaxZ = solution.distanceMax(Piece.ORIGINE, Axe.Profondeur) + 1;
		
		// calcul la distance max sur les axes de la solution
		int distanceMinX = solution.distanceMin(Piece.ORIGINE, Axe.Abscisse) - 1;
		int distanceMinY = solution.distanceMin(Piece.ORIGINE, Axe.Ordonnee) - 1;
		int distanceMinZ = solution.distanceMin(Piece.ORIGINE, Axe.Profondeur) - 1;
		
		// Effectue toutes les rotation sur tout les angles
		// Par rapport au centre
		for (; pieceSolutionProgress.angleX < 4; pieceSolutionProgress.angleX++) {
			for (; pieceSolutionProgress.angleY < 4; pieceSolutionProgress.angleY++) {
				for (; pieceSolutionProgress.angleZ < 4; pieceSolutionProgress.angleZ++) {
					Piece clone = null;
					try {
						clone = (Piece) piece.clone();
					} catch (CloneNotSupportedException e) {
					}
					for (; pieceSolutionProgress.x < distanceMaxX; pieceSolutionProgress.x++) {
						for (; pieceSolutionProgress.y < distanceMaxY; pieceSolutionProgress.y++) {
							for (; pieceSolutionProgress.z < distanceMaxZ; pieceSolutionProgress.z++) {
								clone.moveTo(new Cube(pieceSolutionProgress.x, pieceSolutionProgress.y, pieceSolutionProgress.z));
								// Si la pice est incluse alors c'est une
								// solution
								if (solution.include(clone)) {
									// Si la pice n'existe pas dj
									if (!allPieceSolutions.contains(clone)) {
										if (logger.isDebugEnabled()) {
											logger
													.debug("La pice "
															+ clone
															+ " est incluse dans la solution "
															+ solution);
										}
										allPieceSolutions.add(clone);
										return;
									}
								}
							}
							pieceSolutionProgress.z = distanceMinZ;
						}
						pieceSolutionProgress.y = distanceMinY;
					}
					pieceSolutionProgress.x = distanceMinX;
					piece.rotation(Axe.Profondeur, 1);
				}
				pieceSolutionProgress.angleZ = 0;
				piece.rotation(Axe.Ordonnee, 1);
			}
			pieceSolutionProgress.angleY = 0;
			piece.rotation(Axe.Abscisse, 1);
		}
		
		logger.info("Piece " + piece.getNom() + " "
				+ allPieceSolutions.size() + " solutions trouves");
	
		if (allPieceSolutions.size() == 0)
			throw new PuzzleInvalideException("Piece " + piece.getNom() + " 0 solution trouve");
	}
	
	/**
	 * Dcrit l'avancement dans la recherche de combinaison possibles pour cette pices
	 */
	protected static class PieceSolutionProgress
	{
		int 
		angleX = 0,
		angleY = 0,
		angleZ = 0,
		x = BORNE_MIN,
		y = BORNE_MIN,
		z = BORNE_MIN;
		
		/**
		 * 
		 * @return
		 */
		public boolean rechercheTerminee() {
			return angleX == 4;
		}
	}
}
