/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core.engine.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import fr.hypik.developpez.defis.soma.core.exception.PuzzleInvalideException;
import fr.hypik.developpez.defis.soma.core.listener.ISomaEngineProgressListener;
import fr.hypik.developpez.defis.soma.core.model.Axe;
import fr.hypik.developpez.defis.soma.core.model.Cube;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * Implmentation du moteur de resolution suivant le premier algorithme dcrit
 * ici: http://web.me.com/rouxjeanbernard/Site/AM/html/amch94.html
 * 
 * @author Pierrick HYMBERT.
 * 
 */
public class SimpleSomaEngine extends AbstractSomaEngine {

	/**
	 * Le logger de la classe
	 */
	private static final Logger logger = Logger
			.getLogger(SimpleSomaEngine.class);
	/**
	 * L'instance de SimpleSomaEngine
	 */
	private static final SimpleSomaEngine singleton = new SimpleSomaEngine();

	/**
	 * @return L'instance de SimpleSomaEngine
	 */
	public static SimpleSomaEngine get() {
		return singleton;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * fr.hypik.developpez.defis.soma.core.core.engine.ISomaEngine#getAllSolutions
	 * (fr. hypik.developpez.defis.soma.model.Cube[])
	 */
	public List<Piece[]> getAllSolutions(Cube[] soma)
			throws PuzzleInvalideException {
		// Dmmare la recherche de solutions
		start();
		
		// Le puzzle  atteindre
		Piece solution = new Piece("Soma", soma);

		logger.info("Recherche toutes les solutions pour le puzzle: "
				+ solution);

		// Toutes les positions de toutes les pices dans la solution
		Map<Integer, List<Piece>> allSolutionsByPiece = new HashMap<Integer, List<Piece>>();

		// Recherche toutes les positions de toutes les pices dans la solution
		buildAllSolutionsByPiece(solution, allSolutionsByPiece);
		
		// Rearange l'ordre des pices
		allSolutionsByPiece = orderSomaPieces(allSolutionsByPiece);

		// Le nombre de solutions testees
		long totalSolutionTestees = 0;

		// Le nombre de solutions testees
		long solutionTestees = 0;

		// La liste de toutes les solutions
		List<Piece[]> allSolutions = new ArrayList<Piece[]>();

		// La solution courante
		Map<Integer, Integer> solutionPiecesCourante = new HashMap<Integer, Integer>();

		// Les solutions dj utilisees
		List<Map<Integer, Integer>> solutionInterdites = new ArrayList<Map<Integer, Integer>>();

		// Recherche toutes les solutions
		try {
			while ((solutionTestees = getSolution(solution,
					allSolutionsByPiece, solutionPiecesCourante,
					solutionInterdites)) > 0) {
				
				// La nouvelle map des solutions
				Map<Integer, Integer> solutionPiecesCouranteBis = new HashMap<Integer, Integer>();

				// Incrmente le nombre total de solution testees
				totalSolutionTestees += solutionTestees;

				// Construit la solution
				Piece[] finalSolution = new Piece[somaPieces.length];
				for (int i = 0; i < somaPieces.length; i++) {
					Piece piece = allSolutionsByPiece.get(i).get(
							solutionPiecesCourante.get(i));
					finalSolution[i] = piece;
					solutionPiecesCouranteBis.put(i, solutionPiecesCourante
							.get(i));
					logger.info(piece);
					if (logger.isDebugEnabled()) {
						logger.debug("Piece " + piece.getNom() + " solution "
								+ solutionPiecesCourante.get(i) + "/"
								+ allSolutionsByPiece.get(i).size());
					}
				}

				// Incrmente la dernire combinaison
				solutionPiecesCourante.put(somaPieces.length - 1,
						solutionPiecesCourante.get(somaPieces.length - 1) + 1);

				// Ajoute la solution  la liste des solutions
				allSolutions.add(finalSolution);

				// Ajoute la solution courante  la liste des solutions
				solutionInterdites.add(solutionPiecesCouranteBis);

				logger.info("Nombre de solutions trouves: "
						+ allSolutions.size());
				
				// Vrifie que la recherche n'est pas interropue
				if( isInterrupted() )
				{
					logger.info("Recherche interrompue");
					return allSolutions;
				}

				// Previent les listeners
				for (ISomaEngineProgressListener listener : listeners)
					listener.solutionTrouvee(finalSolution, solution);
			}
		} catch (PuzzleInvalideException e) {
			// Lve l'exception si on a trouve aucune solution
			if (allSolutions.size() == 0)
				throw e;
		}

		// Log le nombre de solution trouves
		logger.info("Nombre de solutions trouves: " + allSolutions.size()
				+ ", nombre de combinaisons testes: " + totalSolutionTestees);

		return allSolutions;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * fr.hypik.developpez.defis.soma.core.core.engine.ISomaEngine#getSolution
	 * (fr.hypik .developpez.defis.soma.model.Cube[])
	 */
	public Piece[] getSolution(Cube[] soma) throws PuzzleInvalideException {
		// Dmmare la recherche de solutions
		start();
		
		Piece solution = new Piece("Soma", soma);

		logger.info("Recherche une solution pour le puzzle: " + solution);

		// Toutes les positions de toutes les pices dans la solution
		Map<Integer, List<Piece>> allSolutionsByPiece = new HashMap<Integer, List<Piece>>();

		// La solution courante
		Map<Integer, Integer> solutionPiecesCourante = new HashMap<Integer, Integer>();

		// Recherche toutes les positions de toutes les pices dans la solution
		buildAllSolutionsByPiece(solution, allSolutionsByPiece);
		
		// Rearange l'ordre des pices
		//allSolutionsByPiece = orderSomaPieces(allSolutionsByPiece);

		// Recherche une solution
		Piece[] finalSolution = null;

		getSolution(solution, allSolutionsByPiece, solutionPiecesCourante, null);

		// Construit la solution
		finalSolution = new Piece[somaPieces.length];
		for (int i = 0; i < somaPieces.length; i++) {
			Piece piece = allSolutionsByPiece.get(i).get(
					solutionPiecesCourante.get(i));
			finalSolution[i] = piece;
			logger.info(piece);
			if (logger.isDebugEnabled()) {
				logger.debug("Piece " + piece.getNom() + " solution "
						+ solutionPiecesCourante.get(i) + "/"
						+ allSolutionsByPiece.get(i).size());
			}
		}
		
		// Vrifie que la recherche n'est pas interropue
		if( isInterrupted() )
		{
			logger.info("Recherche interrompue");
			return finalSolution;
		}

		// Previent les listeners
		for (ISomaEngineProgressListener listener : listeners)
			listener.solutionTrouvee(finalSolution, solution);

		return finalSolution;
	}

	/**
	 * Recherche une solution
	 * 
	 * @param solution
	 *            Le puzzle a atteindre
	 * @param allSolutionsByPiece
	 *            Toutes les positions de toutes les pices dans la solution
	 * @param solutionPiecesCourante
	 *            La solution courante. La cl est l'index de la pice . La
	 *            valeur est l'index de la solution pour cette pice dans la
	 *            liste de toutes les solution de la piece
	 * @param solutionInterdites
	 *            La liste des combinaisons interdites
	 * @return Le nombre de solutions testes
	 * @throws PuzzleInvalideException
	 *             Si le puzzle est irralisable
	 */
	protected long getSolution(Piece solution,
			Map<Integer, List<Piece>> allSolutionsByPiece,
			Map<Integer, Integer> solutionPiecesCourante,
			List<Map<Integer, Integer>> solutionInterdites)
			throws PuzzleInvalideException {

		// Le nombre de solution testees
		long nbSolutionsTestees = 0;

		// Le nom de la pice courante  placer
		int pieceCourante = Math.max(0, solutionPiecesCourante.size() - 1);

		// Definie la premire solution pour la premire pice
		if (!solutionPiecesCourante.containsKey(pieceCourante))
			solutionPiecesCourante.put(pieceCourante, 0);

		// Fixe si la dernire solution tait la dernire combinaison de la
		// dernire pice
		while (solutionPiecesCourante.containsKey(pieceCourante)
				&& solutionPiecesCourante.get(pieceCourante) == allSolutionsByPiece
						.get(pieceCourante).size()) {
			pieceCourante--;
		}

		// Vrifie s'il y a des solutions
		long maxCombinaisonsRestantes = computeSolutionsRestante(pieceCourante,
				solutionPiecesCourante, allSolutionsByPiece);
		if (maxCombinaisonsRestantes == 0) {
			throw new PuzzleInvalideException();
		}

		// Log l'entre dans la mthode de recherche de solution
		logger.info("Recherche d'une solution sur un ensemble de "
				+ maxCombinaisonsRestantes + " combinaisons possibles");

		// Le flag indiquant que l'on a trouv une solution
		boolean solutionTrouvee = false;

		// Tant que l'on n'a pas test toutes les solutions ou pas trouv de
		// solution
		// CAD:
		// Tant que
		// l'indice de la solution de la premire pice n'est pas suprieur au
		// nombre de solution de la premire pice
		while (solutionPiecesCourante.get(0) < allSolutionsByPiece.get(0)
				.size()) {
			
			// Vrifie que la recherche n'est pas interropue
			if( isInterrupted() )
			{
				logger.info("Recherche interrompue");
				return nbSolutionsTestees;
			}
			
			try {
				// L'indice de la solution de la pice courante
				int solutionPieceCourante = solutionPiecesCourante
						.get(pieceCourante);

				// Toutes les solutions de la pice courante
				List<Piece> allSolutionsPiece = allSolutionsByPiece
						.get(pieceCourante);

				// La solution de la pice courante
				Piece piece = allSolutionsPiece.get(solutionPieceCourante);

				// Log l'avancement
				if (logger.isDebugEnabled()) {
					logger.debug("Recherche une solution pour la pice "
							+ piece.getNom() + "(" + pieceCourante + "/"
							+ somaPieces.length + ") " + solutionPieceCourante
							+ "/" + allSolutionsPiece.size()
							+ ", solutions testes " + nbSolutionsTestees
							+ " reste " + maxCombinaisonsRestantes);
				}

				// Incrmente le nombre de solutions testes
				nbSolutionsTestees++;

				// Calcul le nombre de solutions restantes
				maxCombinaisonsRestantes = computeSolutionsRestante(
						pieceCourante, solutionPiecesCourante,
						allSolutionsByPiece);

				// Check si toutes les pices sont incluses
				// Ou si la solution existe dej
				if (collisionExiste(allSolutionsByPiece,
						solutionPiecesCourante, pieceCourante)
						|| (pieceCourante + 1 == somaPieces.length
								&& solutionInterdites != null && solutionInterdites
								.contains(solutionPiecesCourante))) {
					// Le puzzle n'est pas terminee
					// Soit on passe  la solution suivante soit on recule
					if (++solutionPieceCourante == allSolutionsPiece.size()
							&& pieceCourante > 0) {
						do {							
							// Vrifie que la recherche n'est pas interropue
							if( isInterrupted() )
							{
								logger.info("Recherche interrompue");
								return nbSolutionsTestees;
							}
							
							// On reinitialize la solution de la pice suivante
							//  zero
							solutionPieceCourante = 0;

							// Set la nouvelle position de cette pice dans les
							// solutions  tester
							solutionPiecesCourante.put(pieceCourante,
									solutionPieceCourante);

							// On retourne  la pice prcdente
							pieceCourante--;

							// Incrmente les solutions de la pice prcdente
							solutionPieceCourante = solutionPiecesCourante
									.get(pieceCourante) + 1;

							// Log le recul
							if (logger.isDebugEnabled()) {
								logger
										.debug("Toutes les solutions pour la pice "
												+ piece.getNom()
												+ "("
												+ (pieceCourante - 1)
												+ "/"
												+ somaPieces.length
												+ ") ont t teste, retour  la pice prcdente");
							}
						} while (solutionPieceCourante == allSolutionsByPiece
								.get(pieceCourante).size()
								&& pieceCourante > 0);
					}
					// Set la nouvelle position de cette pice dans les
					// solutions  tester
					solutionPiecesCourante.put(pieceCourante,
							solutionPieceCourante);
				} else {
					// Vrifie si on a termin le puzzle
					if (pieceCourante + 1 == somaPieces.length) {
						solutionTrouvee = true;
						break;
					} else {
						// On passe  la pice suivante
						pieceCourante++;

						// Initialise la pice courante
						solutionPiecesCourante.put(pieceCourante, 0);
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
				logger.error(e, e);
			}
		}

		maxCombinaisonsRestantes = computeSolutionsRestante(pieceCourante,
				solutionPiecesCourante, allSolutionsByPiece);

		// Log la fin de l'algo
		logger.info("Puzzle test: " + solution);
		logger
				.info("Algorithme de recherche simple terminee, solution trouvee ? "
						+ solutionTrouvee);
		logger.info("Combinaisons testes: " + nbSolutionsTestees
				+ ", combinaisons restantes: " + maxCombinaisonsRestantes);

		// Lve l'exception solution non trouvee
		if (!solutionTrouvee) {
			throw new PuzzleInvalideException();
		}

		// Retourne le nombre de solution testees
		return nbSolutionsTestees;
	}

	/**
	 * Calcul le nombre de solutions restantes
	 * 
	 * @param pieceCourante
	 * @param solutionPiecesCourante
	 *            La solution courante
	 * @param allSolutionsByPiece
	 *            L'ensemble de toutes les solutions
	 * @return Le nombre de solutions potentiellement restante
	 */
	private long computeSolutionsRestante(int pieceCourante,
			Map<Integer, Integer> solutionPiecesCourante,
			Map<Integer, List<Piece>> allSolutionsByPiece) {
		long maxSolutionsPossibles = 1;
		for (int i = 0; i < somaPieces.length; i++) {
			int solutionRestantesPieceCourante = solutionPiecesCourante
					.containsKey(i)
					&& i <= pieceCourante ? solutionPiecesCourante.get(i) : 0;
			List<Piece> allPieceSolutions = allSolutionsByPiece.get(i);
			maxSolutionsPossibles *= allPieceSolutions.size()
					- solutionRestantesPieceCourante;
		}

		return maxSolutionsPossibles;
	}

	/**
	 * Vrifie si une collision existe dans la solution actuelle Si aucune pice
	 * est en collision, alors le puzzle est termine
	 * 
	 * @param allSolutionsByPiece
	 *            Toutes les positions de toutes les pices dans la solution
	 * @param solutionPiecesCourante
	 *            La solution courante
	 * @param pieceCourante
	 *            L'indice de la pice courante
	 * @return
	 */
	protected boolean collisionExiste(
			Map<Integer, List<Piece>> allSolutionsByPiece,
			Map<Integer, Integer> solutionPiecesCourante, int pieceCourante) {

		Piece nouvellePiece = allSolutionsByPiece.get(pieceCourante).get(
				solutionPiecesCourante.get(pieceCourante));

		for (int i = 0; i < pieceCourante; i++) {
			// La pice a tester
			Piece pieceI = allSolutionsByPiece.get(i).get(
					solutionPiecesCourante.get(i));

			if (pieceI.collide(nouvellePiece)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Construit les solutions possibles de toutes les pices dans la structure
	 * 
	 * @param solution
	 *            La solution
	 * @param allSolutionsByPiece
	 *            Toutes les solutions des pces  l'interieur de la structure
	 * @return Le nombre maximum de solution
	 * @throws PuzzleInvalideException
	 *             Si le puzzle est invalide
	 */
	protected void buildAllSolutionsByPiece(Piece solution,
			Map<Integer, List<Piece>> allSolutionsByPiece)
			throws PuzzleInvalideException {		
		for (int i = 0; i < this.somaPieces.length; i++) {
			List<Piece> allPieceSolutions = buildPieceSolutions(this.somaPieces[i],
					solution);
			allSolutionsByPiece.put(i, allPieceSolutions);
		}
	}
	
	/**
	 * 
	 * @param allSolutionsByPiece
	 * @return
	 */
	protected Map<Integer, List<Piece>> orderSomaPieces(Map<Integer, List<Piece>> allSolutionsByPiece)
	{
		Map<Integer, List<Piece>> allSolutionsByPieceOrdered = new HashMap<Integer, List<Piece>>();
		
		SortedMap<Integer, List<Integer>> pieceIndexBySolutionsPossible = new TreeMap<Integer, List<Integer>>();
		
		// Tri les solutions
		for(Map.Entry<Integer, List<Piece>> entry : allSolutionsByPiece.entrySet())
		{
			List<Integer> indexes = pieceIndexBySolutionsPossible.get(entry.getValue().size());
			if( indexes == null )
			{
				indexes = new ArrayList<Integer>();
				pieceIndexBySolutionsPossible.put(entry.getValue().size(), indexes);
			}
			indexes.add(entry.getKey());
			pieceIndexBySolutionsPossible.put(entry.getValue().size(), indexes);
		}
		
		// Place les pices dans un sens intelligent
		// Et place les autres dans l'ordre croissant
		Piece[] somaPieces = new Piece[this.somaPieces.length];
		
		// Rcupre les indexes
		List<List<Integer>> indexess = new ArrayList<List<Integer>>(pieceIndexBySolutionsPossible.values());
		
		// Met la pice qui a le plus de solution en premier
		List<Integer> firstIndexes = indexess.get(indexess.size() - 1);
		int j = 0;
		somaPieces[j] = this.somaPieces[firstIndexes.get(0)];
		allSolutionsByPieceOrdered.put(j, allSolutionsByPiece.get(firstIndexes.get(0)));
		j++;
		
		for(int i = 0; i < indexess.size() - 1; i++)
		{
			List<Integer> indexes = indexess.get(i);
			for(Integer index : indexes)
			{
				somaPieces[j] = this.somaPieces[index];
				allSolutionsByPieceOrdered.put(j, allSolutionsByPiece.get(index));
				j++;
			}
		}
		for(int k = 1; k < firstIndexes.size(); k++)
		{
			int index = firstIndexes.get(k);
			somaPieces[j] = this.somaPieces[index];
			allSolutionsByPieceOrdered.put(j, allSolutionsByPiece.get(index));
			j++;
		}
		this.somaPieces = somaPieces;
		
		// Log le nouvel ordre des pices est
			StringBuffer buffer = new StringBuffer();
			buffer.append("Nouvel ordre des pices tabli: ");
			// Tri les solutions
			for(Piece piece : this.somaPieces)
			{
				buffer.append(piece.getNom());
				buffer.append(",");
			}
			logger.info(buffer);
		
		return allSolutionsByPieceOrdered;
	}

	/**
	 * Recherche toutes les de la piece dans la solution
	 * 
	 * @param piece
	 *            La pice  placer
	 * @param solution
	 *            La solution
	 * @return Toutes les solutions de la piece dans la solution
	 * @throws PuzzleInvalideException
	 *             Si le puzzle est invalide
	 */
	protected List<Piece> buildPieceSolutions(Piece piece, Piece solution)
			throws PuzzleInvalideException {
		List<Piece> allPieceSolutions = new ArrayList<Piece>();

		if (logger.isDebugEnabled()) {
			logger.debug("Recherche des solutions pour la piece "
					+ piece.getNom());
		}
		
		// calcul la distance max sur les axes de la solution
		int distanceMaxX = solution.distanceMax(Piece.ORIGINE, Axe.Abscisse) + 1;
		int distanceMaxY = solution.distanceMax(Piece.ORIGINE, Axe.Ordonnee) + 1;
		int distanceMaxZ = solution.distanceMax(Piece.ORIGINE, Axe.Profondeur) + 1;
		
		// calcul la distance max sur les axes de la solution
		int distanceMinX = solution.distanceMin(Piece.ORIGINE, Axe.Abscisse) - 1;
		int distanceMinY = solution.distanceMin(Piece.ORIGINE, Axe.Ordonnee) - 1;
		int distanceMinZ = solution.distanceMin(Piece.ORIGINE, Axe.Profondeur) - 1;

		for (int angleX = 0; angleX < 4; angleX++) {
			for (int angleY = 0; angleY < 4; angleY++) {
				for (int angleZ = 0; angleZ < 4; angleZ++) {
					Piece clone = null;
					try {
						clone = (Piece) piece.clone();
					} catch (CloneNotSupportedException e) {
					}
					for (int x = distanceMinX; x < distanceMaxX; x++) {
						for (int y = distanceMinY; y < distanceMaxY; y++) {
							for (int z = distanceMinZ; z < distanceMaxZ; z++) {
								clone.moveTo(new Cube(x, y, z));
								// Si la pice est incluse alors c'est une
								// solution
								if (solution.include(clone)) {
									// Si la pice n'existe pas dj
									if (!allPieceSolutions.contains(clone)) {
										if (logger.isDebugEnabled()) {
											logger
													.debug("La pice "
															+ clone
															+ " est incluse dans la solution "
															+ solution);
										}
										allPieceSolutions.add(clone);
										try {
											clone = (Piece) clone.clone();
										} catch (CloneNotSupportedException e) {
										}
									}
								}
							}
						}
					}
					piece.rotation(Axe.Profondeur, 1);
				}
				piece.rotation(Axe.Ordonnee, 1);
			}
			piece.rotation(Axe.Abscisse, 1);
		}

		logger.info("Piece " + piece.getNom() + " " + allPieceSolutions.size()
				+ " solution(s) trouve(s)");

		if (allPieceSolutions.size() == 0)
			throw new PuzzleInvalideException("Piece " + piece.getNom()
					+ " 0 solution trouve");

		return allPieceSolutions;
	}
}
