/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core.factory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import fr.hypik.developpez.defis.soma.core.exception.CsvMalformeException;
import fr.hypik.developpez.defis.soma.core.model.Cube;

/**
 * Lecteur csv transformant un fichier CSV en cubes
 * 
 * @author Pierrick HYMBERT
 * 
 */
public class CsvFactory {
	/**
	 * L'instance de CSV factory
	 */
	private static final CsvFactory singleton = new CsvFactory();
	
	/**
	 * @return L'instance de CSV factory
	 */
	public static CsvFactory get()
	{
		return singleton;
	}
	
	/**
	 * Construit un cube depuis une ligne
	 * @param ligne La ligen  convertir
	 * @param noLigne Le numro de la ligne
	 * @return Le cube correspondant
	 * @throws CsvMalformeException Si la lecture de la ligne lve une erreur
	 */
	public Cube readCube(String ligne, int noLigne) throws CsvMalformeException {
		int x,y,z;
		String[] xyz = ligne.split(",");
		if( xyz.length != 3 )
			throw new CsvMalformeException(ligne, noLigne, "Une ligne doit tre compose de trois nombres entiers sparars par des virgules");
		try
		{
			x = Integer.valueOf(xyz[0].trim());
		}
		catch (NumberFormatException e) {
			throw new CsvMalformeException(ligne, noLigne, "Impossible de lire l'abscisse");
		}
		try
		{
			y = Integer.valueOf(xyz[1].trim());
		}
		catch (NumberFormatException e) {
			throw new CsvMalformeException(ligne, noLigne, "Impossible de lire l'ordonne");
		}
		try
		{
			z = Integer.valueOf(xyz[2].trim());
		}
		catch (NumberFormatException e) {
			throw new CsvMalformeException(ligne, noLigne, "Impossible de lire la profondeur");
		}
		return new Cube(x, y, z);
	}
	
	/**
	 * Lit un fichier CSV representant un soma
	 * @param fichier Le fichier  lire
	 * @return Les cubes correspondant au fichier CSV
	 * @throws CsvMalformeException
	 */
	public Cube[] readSoma(File fichier) throws CsvMalformeException
	{
		Cube[] soma = null;
		try {
			soma = readCubes(new FileInputStream(fichier));
		} catch (FileNotFoundException e) {
			throw new CsvMalformeException("", 1, "Une erreur d'accs au fichier est arrive");
		}
		// Verifie le nombre de cube
		if( soma.length != 27 )
			throw new CsvMalformeException("", 1, "Le fichier ne comporte pas 27 cubes...");
		return soma;
	}
	
	/**
	 * Lit un fichier CSV et retourne des cubes
	 * @param fichier Le fichier  lire
	 * @return Les cubes correspondant au fichier CSV
	 * @throws CsvMalformeException
	 */
	public Cube[] readCubes(File fichier) throws CsvMalformeException
	{
		try {
			return readCubes(new FileInputStream(fichier));
		} catch (FileNotFoundException e) {
			throw new CsvMalformeException("", 1, "Une erreur d'accs au fichier est arrive");
		}
	}
	
	/**
	 * Lit un fichier CSV et retourne des cubes
	 * @param stream Le flux  lire
	 * @return Les cubes correspondant au fichier CSV
	 * @throws CsvMalformeException
	 */
	public Cube[] readCubes(InputStream stream) throws CsvMalformeException
	{
		return readCubes(new BufferedReader(new InputStreamReader(stream)));
	}
	
	/**
	 * Lit un fichier CSV et retourne des cubes
	 * @param reader Le lecteur du fichier CSV
	 * @return Les cubes correspondant au fichier CSV
	 * @throws CsvMalformeException
	 */
	public Cube[] readCubes(BufferedReader reader) throws CsvMalformeException
	{
		List<Cube> cubes = new ArrayList<Cube>();
		
		String ligne = null;
		int noLigne = 1;
		try {
			while( ( ligne = reader.readLine()) != null )
			{
				// Vrifie si la ligne n'est pas un comentaire
				if( !ligne.startsWith("#") && !ligne.startsWith("/") )
					cubes.add(readCube(ligne, noLigne));
				noLigne ++;
			}
		} catch (IOException e) {
			throw new CsvMalformeException(ligne, noLigne, "Une erreur d'accs au flux est arrive");
		}
		finally
		{
			try {
				reader.close();
			} catch (IOException e) {
				throw new CsvMalformeException(ligne, noLigne, "Une erreur d'accs au flux est arrive");
			}
		}
		
		return cubes.toArray(new Cube[0]);
	}
}
