/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.core.factory;

import org.apache.log4j.Logger;

import fr.hypik.developpez.defis.soma.core.exception.CsvMalformeException;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * Le constructeur des 7 pices du soma
 * 
 * @author Pierrick HYMBERT
 */
public class PieceFactory {
	/**
	 * Le logger de la classe
	 */
	private static final Logger logger = Logger.getLogger(PieceFactory.class);
	
	/**
	 * L'instance de Piece factory
	 */
	private static final PieceFactory singleton = new PieceFactory();
	
	/**
	 * @return L'instance de Piece factory
	 */
	public static PieceFactory get()
	{
		return singleton;
	}
	/**
	 * Retourne une piece de soma selon son nom
	 * 
	 * @param nom
	 *            Le nom de la piece
	 * @return La piece
	 * @throws CsvMalformeException Si un problme survient durant le chargement de la pice
	 */
	public Piece loadSomaPiece(String nom) throws CsvMalformeException {
		if(logger.isDebugEnabled())
			logger.debug("Chargement de la piece " + nom);
		Piece piece = new Piece(nom, CsvFactory.get()
				.readCubes(
						getClass().getResourceAsStream(
								"/pieces/" + nom)));
		if(logger.isDebugEnabled())
			logger.debug("Piece chargee" + piece);
		return piece;
	}
	
	/**
	 * Retourne toutes les pices de SOMA
	 * @return
	 * @throws CsvMalformeException  Si un problme survient durant le chargement des pices
	 */
	public Piece[] getAllSomaPieces() throws CsvMalformeException
	{
		return new Piece[]{	
				 loadSomaPiece("A")	
				,loadSomaPiece("B")	
				,loadSomaPiece("P")
				,loadSomaPiece("L")
				,loadSomaPiece("T")
				,loadSomaPiece("Z")
				,loadSomaPiece("V")
		};
	}
}
