/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma;

import java.awt.Frame;
import java.io.File;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;

import fr.hypik.developpez.defis.soma.core.engine.impl.SimpleSomaEngine;
import fr.hypik.developpez.defis.soma.core.exception.CsvMalformeException;
import fr.hypik.developpez.defis.soma.core.exception.PuzzleInvalideException;
import fr.hypik.developpez.defis.soma.core.factory.CsvFactory;
import fr.hypik.developpez.defis.soma.core.factory.PieceFactory;
import fr.hypik.developpez.defis.soma.core.listener.ISomaEngineProgressListener;
import fr.hypik.developpez.defis.soma.core.model.Piece;
import fr.hypik.developpez.defis.soma.fenetre.SomaFenetre;
import fr.hypik.developpez.defis.soma.view.Java3DView;

/**
 * La classe qui gre l'application
 * @author Pierrick HYMBERT
 *
 */
public class Application {
	/**
	 * Le logger de la classe
	 */
	private static final Logger logger = Logger.getLogger(Application.class);
	
	/**
	 * Le puzzle courant
	 */
	private Piece puzzle;
	
	/**
	 * La fentre de l'application
	 */
	private SomaFenetre somaFenetre;
	
	/**
	 * La vue de gauche
	 */
	private Java3DView java3dLeftView;
	
	/**
	 * La vue de droite
	 */
	private Java3DView java3dRightView;
	
	/**
	 * L'ensemble des solutions
	 */
	private ArrayList<Piece[]> solutions = new ArrayList<Piece[]>();
	
	/**
	 * La dernire solution affiche
	 */
	private int lastSolution = -2;
	
	/**
	 * La solution courante
	 */
	private int solutionCourante = -1;
	
	/**
	 * Le thread de recherche de solutions
	 */
	Thread engineThread = null;
	
	/**
	 * @throws Exception 
	 * 
	 */
	public Application() throws Exception {
		logger.info("Dmarage de l'application");
		
		logger.debug("Creation de l'ihm");
		somaFenetre = new SomaFenetre();
		if( ApplicationAppender.get() != null )
			ApplicationAppender.get().setText(somaFenetre.getLogTextArea());
		else
			logger.warn("Le logger dans la console de text ne fonctionnera pas s'il n'est pas dclarer dans log4j.properties");
		
		// Cration des ponts awt
		Composite compositeLeft = new Composite(somaFenetre.getGroupLeft(), SWT.EMBEDDED);
		compositeLeft.setLayout(new FillLayout( ));
		Frame leftAwtFrame = SWT_AWT.new_Frame(compositeLeft);		
		java3dLeftView = new Java3DView(leftAwtFrame);
		
		Composite compositeRight = new Composite(somaFenetre.getGroupRight(), SWT.EMBEDDED);
		compositeRight.setLayout(new FillLayout( ));
		Frame rightAwtFrame = SWT_AWT.new_Frame(compositeRight);
		java3dRightView = new Java3DView(rightAwtFrame);
		
		logger.debug("IHM construite");
		
		// Creation des listeners
		SimpleSomaEngine.get().addSomaEngineProgressListener(getSomaEngineProgressListener());
		somaFenetre.getFileExitItem().addSelectionListener(getFileExitItemSelectionListener());
		somaFenetre.getFileOpenItem().addSelectionListener(getFileOpenItemSelectionListener());		
		somaFenetre.getTrouverSolution().addSelectionListener(getTrouverSolutionSelectionListener());
		for(Button b : somaFenetre.getPiecesVisibleButton())
			b.addSelectionListener(getShowPieceSelectionListener());
		for(MenuItem menuItem : somaFenetre.getPuzzlesMenuItem())
			menuItem.addSelectionListener(getOpenPuzzleItemSelectionListener());
		somaFenetre.getTrouverToutesSolutions().addSelectionListener(getTrouverToutesSolutionsSelectionListener());
		somaFenetre.getSolutionPrecedente().addSelectionListener(getSolutionPrecedenteSelectionListener());
		somaFenetre.getArreterRechercheSolutions().addSelectionListener(getArreterRechercheSolutionsSelectionListener());
		somaFenetre.getSolutionSuivante().addSelectionListener(getSolutionSuivanteSelectionListener());
		
		// Ouvre la fentre
		somaFenetre.getsShell().open();
		
		// Dmarrer le thread de lecture des logs
		
		//Fix pour l'include awt
		somaFenetre.getsShell().layout(true, true);
		
		// Gestion des evenements SWT
		while( !somaFenetre.getsShell().isDisposed() )
		{
			if( !somaFenetre.getsShell().getDisplay().readAndDispatch() )
				somaFenetre.getsShell().getDisplay().sleep();
		}
	}
	
	/**
	 * @return Le responsable de l'arret de la recherche de solutions
	 */
	private SelectionListener getArreterRechercheSolutionsSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				// Arrete une execution prcdente
				if( engineThread != null && engineThread.isAlive() )
				{
					SimpleSomaEngine.get().interrupt();
					try {
						engineThread.join();
					} catch (InterruptedException e) {}
				}
				
				// Met  jour les boutons
				somaFenetre.getTrouverSolution().setEnabled(true);
				somaFenetre.getTrouverToutesSolutions().setEnabled(true);
				somaFenetre.getArreterRechercheSolutions().setEnabled(false);
				
				// Affiche la solution
				displaySolution();
			}
		};
	}

	/**
	 * @return Le responsable de la prise en compte des solutions trouves
	 */
	private ISomaEngineProgressListener getSomaEngineProgressListener() {
		return new ISomaEngineProgressListener() {			
			public void solutionTrouvee(Piece[] solution, Piece puzzle) {
				solutions.add(solution);
				somaFenetre.getsShell().getDisplay().syncExec(new Runnable() {					
					public void run() {
						displaySolution();
					}
				});
			}
		};
	}

	/**
	 * @return Affiche la solution precedente
	 */
	private SelectionListener getSolutionPrecedenteSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				// Affiche la solution prcdente
				solutionCourante--;
				displaySolution();
			}
		};
	}
	
	/**
	 * @return Affiche la solution precedente
	 */
	private SelectionListener getSolutionSuivanteSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				// Affiche la solution suivante
				solutionCourante++;
				displaySolution();
			}
		};
	}
	
	/**
	 * @return Le responsable de la fermeture de l'application
	 */
	private SelectionListener getFileExitItemSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				somaFenetre.getsShell().close();
			}
		};
	}
	
	/**
	 * @return Le responsable de la fermeture de l'application
	 */
	private SelectionListener getShowPieceSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				Button b = (Button) selectionEvent.widget;
				String piece = ((Button)selectionEvent.widget).getText();
				java3dRightView.setPieceVisible(piece, b.getSelection());
			}
		};
	}
	
	/**
	 * @return Le responsable de la fermeture de l'application
	 */
	private SelectionListener getTrouverSolutionSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				// Arrete une execution prcdente
				if( engineThread != null && engineThread.isAlive() )
				{
					SimpleSomaEngine.get().interrupt();
					try {
						engineThread.join();
					} catch (InterruptedException e) {}
				}
				
				// Donne les pices de soma au moteur
				try {
					SimpleSomaEngine.get().setSomaPieces(PieceFactory.get().getAllSomaPieces());
				} catch (CsvMalformeException e) {
					logger.error("Impossible de charger les pices du cube de soma: " + e, e);
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
					mb.setText("Erreur inattendue");
					mb.setMessage("Impossible de charger les pices du cube de soma");
					mb.open();
					return;
				}
				
				// Vrifie s'il y a un puzzle
				if( puzzle == null )
				{
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
					mb.setText("Aucun puzzle selectionn");
					mb.setMessage("Veuillez selectionner un puzzle");
					mb.open();
					return;
				}
				
				// Vide l'ensemble des solutions prcdentes
				lastSolution = -1;
				solutionCourante = 0;
				solutions.clear();
				
				// Met  jour les boutons
				somaFenetre.getTrouverSolution().setEnabled(false);
				somaFenetre.getTrouverToutesSolutions().setEnabled(false);
				somaFenetre.getArreterRechercheSolutions().setEnabled(true);
				somaFenetre.getSolutionPrecedente().setEnabled(false);
				somaFenetre.getSolutionSuivante().setEnabled(false);
				
				// Creer un thread d'execution
				engineThread = new Thread(){
					public void run() {
						try {
							SimpleSomaEngine.get().getSolution(puzzle.getCubes());
							
							somaFenetre.getsShell().getDisplay().asyncExec(new Runnable() {								
								public void run() {
									
									// Met  jour les boutons
									somaFenetre.getTrouverSolution().setEnabled(true);
									somaFenetre.getTrouverToutesSolutions().setEnabled(true);
									somaFenetre.getArreterRechercheSolutions().setEnabled(false);
									
									displaySolution();
								}
							});
						} catch (final PuzzleInvalideException e) {}
					}
				};
				
				// Dmarre la recherche
				engineThread.start();
			}
		};
	}
	

	
	/**
	 * @return Le responsable de la fermeture de l'application
	 */
	private SelectionListener getTrouverToutesSolutionsSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				// Arrete une execution prcdente
				if( engineThread != null && engineThread.isAlive() )
				{
					SimpleSomaEngine.get().interrupt();
					try {
						engineThread.join();
					} catch (InterruptedException e) {}
				}
				
				// Donne les pices de soma au moteur
				try {
					SimpleSomaEngine.get().setSomaPieces(PieceFactory.get().getAllSomaPieces());
				} catch (CsvMalformeException e) {
					logger.error("Impossible de charger les pices du cube de soma: " + e, e);
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
					mb.setText("Erreur inatendue");
					mb.setMessage("Impossible de charger les pices du cube de soma");
					mb.open();
					return;
				}
				
				// Vrifie s'il y a un puzzle
				if( puzzle == null )
				{
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
					mb.setText("Aucun puzzle selectionn");
					mb.setMessage("Veuillez selectionner un puzzle");
					mb.open();
					return;
				}
				
				// Vide l'ensemble des solutions prcdentes
				lastSolution = -1;
				solutionCourante = 0;
				solutions.clear();
				
				// Met  jour les boutons
				somaFenetre.getTrouverSolution().setEnabled(false);
				somaFenetre.getTrouverToutesSolutions().setEnabled(false);
				somaFenetre.getArreterRechercheSolutions().setEnabled(true);
				somaFenetre.getSolutionPrecedente().setEnabled(false);
				somaFenetre.getSolutionSuivante().setEnabled(false);
				
				// Creer un thread d'execution
				engineThread = new Thread(){
					public void run() {
						try {
							SimpleSomaEngine.get().getAllSolutions(puzzle.getCubes());
							
							somaFenetre.getsShell().getDisplay().asyncExec(new Runnable() {								
								public void run() {
									// Met  jour les boutons
									somaFenetre.getTrouverSolution().setEnabled(true);
									somaFenetre.getTrouverToutesSolutions().setEnabled(true);
									somaFenetre.getArreterRechercheSolutions().setEnabled(false);
									
									displaySolution();
								}
							});
						} catch (final PuzzleInvalideException e) {}
					}
				};
				
				// Dmarre le thread
				engineThread.start();
			}
		};
	}
	
	/**
	 * @return Le responsable de l'ouverture d'un puzzle
	 */
	private SelectionListener getOpenPuzzleItemSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				MenuItem menuItem = (MenuItem) selectionEvent.widget;
				File file = (File) menuItem.getData();
				openPuzzle(file);
			}
		};
	}
	
	/**
	 * @return Le responsable de l'ouverture d'un fichier
	 */
	private SelectionListener getFileOpenItemSelectionListener() {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {}	
			public void widgetSelected(SelectionEvent selectionEvent) {
				FileDialog fileDialog = new FileDialog(somaFenetre.getsShell());				
				String sFile = fileDialog.open();
				if( sFile != null )
				{
					File file = new File(sFile);
					openPuzzle(file);
				}
			}
		};
	}
	
	/**
	 * Ouvre le puzzle en paramtre
	 * @param file Le fichier CSV  ouvrir
	 */
	private void openPuzzle(File file) {
		try
		{
			// Arrete une execution prcdente
			if( engineThread != null && engineThread.isAlive() )
			{
				MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
				mb.setText("Recherche en cours");
				mb.setMessage("Une recherche est en cours!");
				mb.open();
				return;
			}
			
			puzzle = new Piece(file.getName(), CsvFactory.get().readCubes(file));
			
			// Dessine le puzzle
			java3dRightView.clear();
			java3dLeftView.drawPieces(new Piece[]{puzzle}, true);
			
			lastSolution = -1;
			solutionCourante = 0;
			solutions.clear();
			
			somaFenetre.getPuzzleInformationLabel().setText(
					  "Puzzle: " + puzzle.getNom());
			
			somaFenetre.getSolutionPrecedente().setEnabled(false);
			somaFenetre.getSolutionSuivante().setEnabled(false);
			
		}catch (Exception e) {
			logger.error("Une erreur est survenue durant la lecture du fichier CSV: " + e, e);
			MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
			mb.setText("Une erreur est survenue durant la lecture du fichier CSV");
			mb.setMessage(e.getMessage());
			mb.open();
		}
	}
	
	/**
	 * Affiche la solution courante
	 */
	private void displaySolution() {
		somaFenetre.getsShell().getDisplay().asyncExec(new Runnable() {
			public void run() {

				if( solutionCourante <= 0 )
					somaFenetre.getSolutionPrecedente().setEnabled(false);
				else
					somaFenetre.getSolutionPrecedente().setEnabled(true);
				if( solutionCourante + 1 >= solutions.size() )
					somaFenetre.getSolutionSuivante().setEnabled(false);
				else
					somaFenetre.getSolutionSuivante().setEnabled(true);
				
				somaFenetre.getPuzzleInformationLabel().setText(
						  "Puzzle: " + puzzle.getNom()
							+ "\nSolution courante: " + solutionCourante
							+ "\nNombre de solution trouvee: " + solutions.size()
							+ "\nRecherche de solution en cours: " + (engineThread.isAlive() ? "Oui" : "non")
				);
					
				
				if( solutions.size() == 0 )
				{
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_ERROR);
					mb.setText("Aucune solution");
					mb.setMessage("Il n'y a pas de solution");
					mb.open();
					return;
				}
				
				if( solutionCourante < 0 )
				{
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_INFORMATION);
					mb.setText("Premire solution");
					mb.setMessage("La solution affiche est la premire solution");
					mb.open();
					solutionCourante++;
					return;
				}
				
				if( solutionCourante == solutions.size() )
				{
					MessageBox mb = new MessageBox(somaFenetre.getsShell(), SWT.ICON_INFORMATION);
					mb.setText("Dernire solution");
					mb.setMessage("La solution affiche est la dernire solution");
					mb.open();
					solutionCourante--;
					return;
				}
				
				if( lastSolution != solutionCourante )
				{
					// Affiche la solution
					java3dRightView.drawPieces(solutions.get(solutionCourante), false);
					
					// Montre toutes les pices en fonction de la selection
					for(Button b : somaFenetre.getPiecesVisibleButton())
						java3dRightView.setPieceVisible(b.getText(), b.getSelection());
					}
					lastSolution = solutionCourante;
				}
		});
	}
	
	/**
	 * Lance le programme
	 * @param args N/A
	 */
	public static void main(String[] args) {
		try
		{
			new Application();
		}catch (Exception e) {
			logger.error("Fin de l'application  cause de l'erreur: " + e, e);
			System.exit(-1);
		}
		System.exit(0);
	}
}
