package fr.hypik.developpez.defis.soma.fenetre;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;

public class SomaFenetre {

	private Shell sShell = null;  //  @jve:decl-index=0:visual-constraint="10,10"
	private SashForm sashFormTop = null;
	private Group groupLeft = null;
	private Group groupRight = null;
	private TabFolder tabFolder = null;
	private Composite puzzleInformationComposite = null;
	private Composite actionComposite = null;
	private Composite logComposite = null;
	private MenuItem fileOpenItem;
	private MenuItem fileExitItem;
	private Button trouverSolution;
	private SashForm sashForm;
	private Group showPieceGroup;
	private Button[] piecesVisibleButton;
	private MenuItem[] puzzlesMenuItem;
	private Text logTextArea;
	private Button trouverToutesSolutions;
	private Button arreterRechercheSolutions;
	private Button solutionPrecedente;
	private Button solutionSuivante;
	private Label puzzleInformationLabel;
	/**
	 * Creer la fentre de l'application
	 */
	public SomaFenetre() {
		createSShell();
	}
	/**
	 * This method initializes sShell
	 */
	private void createSShell() {
		FillLayout fillLayout = new FillLayout();
		fillLayout.type = org.eclipse.swt.SWT.VERTICAL;
		sShell = new Shell();
		sShell.setText("HyPik - Soma Cube Solver 1.0.0");
		createMenu();
		createSashForm();
		sShell.setLayout(fillLayout);
		sShell.setSize(new Point(729, 412));
	}
	
	private void createMenu() {
		Menu menuBar = new Menu(sShell, SWT.BAR);
		
		MenuItem fileMenuHeader = new MenuItem(menuBar, SWT.CASCADE);
	    fileMenuHeader.setText("&Fichier");
	    
	    Menu fileMenu = new Menu(sShell, SWT.DROP_DOWN);
	    fileMenuHeader.setMenu(fileMenu);

	    fileOpenItem = new MenuItem(fileMenu, SWT.NONE);
	    fileOpenItem.setText("&Ouvrir un csv");

	    fileExitItem = new MenuItem(fileMenu, SWT.NONE);
	    fileExitItem.setText("Quitter");
	    
	    MenuItem puzzlesMenuHeader = new MenuItem(menuBar, SWT.CASCADE);
	    puzzlesMenuHeader.setText("&Puzzles");
	    
	    Menu puzzlesMenu = new Menu(sShell, SWT.DROP_DOWN);
	    puzzlesMenuHeader.setMenu(puzzlesMenu);
	    
	    File puzzlesDir = new File("./puzzles");
	    if( puzzlesDir.exists() )
	    {
	    	List<MenuItem> puzzlesMenuItem = new ArrayList<MenuItem>();
		    for(File puzzle : puzzlesDir.listFiles())
	    	{
	    		if( puzzle.isFile() )
	    		{
	    			// Ajoute une entree dans le menu
	    			MenuItem puzzleMenuItem = new MenuItem(puzzlesMenu, SWT.NONE);
	    			puzzleMenuItem.setText(puzzle.getName());
	    			puzzleMenuItem.setData(puzzle);
	    			puzzlesMenuItem.add(puzzleMenuItem);
	    		}
	    	}
		    this.puzzlesMenuItem = puzzlesMenuItem.toArray(new MenuItem[0]);
	    }
	    
	    sShell.setMenuBar(menuBar);
	}
	/**
	 * This method initializes sashFormTop	
	 *
	 */
	private void createSashForm() {
		sashForm = new SashForm(sShell, SWT.VERTICAL);
		createSashTopForm();
		createTabFolder();
		sashForm.setWeights(new int[]{5, 2});
	}
	
	/**
	 * This method initializes sashFormTop	
	 *
	 */
	private void createSashTopForm() {
		sashFormTop = new SashForm(sashForm, SWT.NONE);
		createGroupLeft();
		createGroupRight();
	}
	/**
	 * This method initializes groupLeft	
	 *
	 */
	private void createGroupLeft() {
		groupLeft = new Group(sashFormTop, SWT.NONE);
		groupLeft.setLayout(new FillLayout());
		groupLeft.setText("Puzzle");
	}
	/**
	 * This method initializes groupRight	
	 *
	 */
	private void createGroupRight() {
		groupRight = new Group(sashFormTop, SWT.NONE);
		groupRight.setLayout(new FillLayout());
		groupRight.setText("Solution");
	}
	/**
	 * This method initializes tabFolder	
	 *
	 */
	private void createTabFolder() {
		tabFolder = new TabFolder(sashForm, SWT.NONE);
		tabFolder.setSize(200, -1);
		createPuzzleInformationComposite();
		createActionComposite();
		createLogComposite();
		TabItem tabItem1 = new TabItem(tabFolder, SWT.NONE);
		tabItem1.setText("Actions");
		tabItem1.setControl(actionComposite);
		TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText("Informations puzzle");
		tabItem.setControl(puzzleInformationComposite);
		TabItem tabItem2 = new TabItem(tabFolder, SWT.NONE);
		tabItem2.setText("Logs");
		tabItem2.setControl(logComposite);
	}
	/**
	 * This method initializes puzzleInformationComposite	
	 *
	 */
	private void createPuzzleInformationComposite() {
		puzzleInformationComposite = new Composite(tabFolder, SWT.NONE);
		puzzleInformationComposite.setLayout(new FillLayout());
		
		puzzleInformationLabel = new Label(puzzleInformationComposite, SWT.NONE);
	}
	/**
	 * This method initializes actionComposite	
	 *
	 */
	private void createActionComposite() {
		actionComposite = new Composite(tabFolder, SWT.NONE);
		actionComposite.setLayout(new GridLayout(6, false));
		
		trouverSolution = new Button(actionComposite, SWT.NONE);
		trouverSolution.setText("Trouver une solution");
		
		trouverToutesSolutions = new Button(actionComposite, SWT.NONE);
		trouverToutesSolutions.setText("Trouver toutes les solutions");
		
		arreterRechercheSolutions = new Button(actionComposite, SWT.NONE);
		arreterRechercheSolutions.setText("Arreter la recherche");
		arreterRechercheSolutions.setEnabled(false);
		
		solutionPrecedente = new Button(actionComposite, SWT.NONE);
		solutionPrecedente.setText("Solution prcdente");
		solutionPrecedente.setEnabled(false);
		
		solutionSuivante = new Button(actionComposite, SWT.NONE);
		solutionSuivante.setText("Solution suivante");
		solutionSuivante.setEnabled(false);
		
		createShowPieceGroup();
	}
	/**
	 * 
	 */
	private void createShowPieceGroup() {
		showPieceGroup = new Group(actionComposite, SWT.NONE);
		showPieceGroup.setLayout(new GridLayout(7, true));
		showPieceGroup.setText("Pices");
		
		piecesVisibleButton = new Button[7];
		
		piecesVisibleButton[0] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[0].setText("A");
		piecesVisibleButton[0].setSelection(true);
		
		piecesVisibleButton[1] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[1].setText("B");
		piecesVisibleButton[1].setSelection(true);
		
		piecesVisibleButton[2] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[2].setText("T");
		piecesVisibleButton[2].setSelection(true);
		
		piecesVisibleButton[3] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[3].setText("Z");
		piecesVisibleButton[3].setSelection(true);
		
		piecesVisibleButton[4] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[4].setText("P");
		piecesVisibleButton[4].setSelection(true);
		
		piecesVisibleButton[5] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[5].setText("L");
		piecesVisibleButton[5].setSelection(true);
		
		piecesVisibleButton[6] = new Button(showPieceGroup, SWT.TOGGLE);
		piecesVisibleButton[6].setText("V");
		piecesVisibleButton[6].setSelection(true);
	}
	/**
	 * This method initializes logComposite	
	 *
	 */
	private void createLogComposite() {
		logComposite = new Composite(tabFolder, SWT.NONE);
		logComposite.setLayout(new FillLayout());
		
		logTextArea = new Text(logComposite, SWT.READ_ONLY | SWT.V_SCROLL | SWT.H_SCROLL | SWT.MULTI);
	}
	/**
	 * @return the sShell
	 */
	public Shell getsShell() {
		return sShell;
	}
	/**
	 * @return the sashFormTop
	 */
	public SashForm getSashForm() {
		return sashFormTop;
	}
	/**
	 * @return the groupLeft
	 */
	public Group getGroupLeft() {
		return groupLeft;
	}
	/**
	 * @return the groupRight
	 */
	public Group getGroupRight() {
		return groupRight;
	}
	/**
	 * @return the tabFolder
	 */
	public TabFolder getTabFolder() {
		return tabFolder;
	}
	/**
	 * @return the puzzleInformationComposite
	 */
	public Composite getProgressionComposite() {
		return puzzleInformationComposite;
	}
	/**
	 * @return the actionComposite
	 */
	public Composite getActionComposite() {
		return actionComposite;
	}
	/**
	 * @return the logComposite
	 */
	public Composite getLogComposite() {
		return logComposite;
	}
	/**
	 * @return the fileOpenItem
	 */
	public MenuItem getFileOpenItem() {
		return fileOpenItem;
	}
	/**
	 * @return the fileExitItem
	 */
	public MenuItem getFileExitItem() {
		return fileExitItem;
	}
	/**
	 * @return the trouverSolution
	 */
	public Button getTrouverSolution() {
		return trouverSolution;
	}
	/**
	 * @return the sashFormTop
	 */
	public SashForm getSashFormTop() {
		return sashFormTop;
	}
	/**
	 * @return the showPieceGroup
	 */
	public Group getShowPieceGroup() {
		return showPieceGroup;
	}
	/**
	 * @return the piecesVisibleButton
	 */
	public Button[] getPiecesVisibleButton() {
		return piecesVisibleButton;
	}
	/**
	 * @return the puzzleInformationComposite
	 */
	public Composite getPuzzleInformationComposite() {
		return puzzleInformationComposite;
	}
	/**
	 * @return the puzzlesMenuItem
	 */
	public MenuItem[] getPuzzlesMenuItem() {
		return puzzlesMenuItem;
	}
	/**
	 * @return the logTextArea
	 */
	public Text getLogTextArea() {
		return logTextArea;
	}
	/**
	 * @return the trouverToutesSolutions
	 */
	public Button getTrouverToutesSolutions() {
		return trouverToutesSolutions;
	}
	/**
	 * @return the solutionPrecedente
	 */
	public Button getSolutionPrecedente() {
		return solutionPrecedente;
	}
	/**
	 * @return the solutionSuivante
	 */
	public Button getSolutionSuivante() {
		return solutionSuivante;
	}
	/**
	 * @return the arreterRechercheSolutions
	 */
	public Button getArreterRechercheSolutions() {
		return arreterRechercheSolutions;
	}
	/**
	 * @return the puzzleInformationLabel
	 */
	public Label getPuzzleInformationLabel() {
		return puzzleInformationLabel;
	}
}
