/**
 * Copyright 2009 Pierrick HYMBERT.
 * Les sources qui constituent ce projet, HyPik-Soma-Cube, 
 * de mme que la documentation associe sont la proprit
 * de leur auteur.
 * Je donne mon accord au site developpez.com pour l'utili-
 * sation de tout ou partie des sources et de la documenta-
 * tion de ce projet dans les pages developpez.com
 */
package fr.hypik.developpez.defis.soma.view;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.media.j3d.Appearance;
import javax.media.j3d.Background;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.Node;
import javax.media.j3d.RenderingAttributes;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseTranslate;
import com.sun.j3d.utils.behaviors.mouse.MouseZoom;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.universe.SimpleUniverse;

import fr.hypik.developpez.defis.soma.core.engine.ISomaEngine;
import fr.hypik.developpez.defis.soma.core.model.Cube;
import fr.hypik.developpez.defis.soma.core.model.Piece;

/**
 * Une vue Java 3D permetant de representer des pices et des cubes
 * 
 * @author Pierrick HYMBERT
 * 
 */
public class Java3DView {

	// L'echelle
	private static final float ECHELLE = .1f;
	
	private static BoundingSphere sphere = new BoundingSphere(new Point3d(), ISomaEngine.BORNE_MAX);
	
	/**
	 * 
	 */
	private static final Map<String, Color3f> colorsByPiece = new HashMap<String, Color3f>();
	
	static
	{
		colorsByPiece.put("A", new Color3f(1, 0, 1));
		colorsByPiece.put("B", new Color3f(0, 1, 1));
		colorsByPiece.put("L", new Color3f(0, 1, 0));
		colorsByPiece.put("P", new Color3f(0.2f, 0.2f, 0.2f));
		colorsByPiece.put("T", new Color3f(0, 0, 1));
		colorsByPiece.put("V", new Color3f(1, 0, 0));
		colorsByPiece.put("Z", new Color3f(1, 1, 0));
	}

	/**
	 * L'objet racine
	 */
	private BranchGroup rootObject;
	private TransformGroup rootTransform;
	private SimpleUniverse universe;
	/**
	 * 
	 */
	private BranchGroup branchGroup;
	
	/**
	 * 
	 */
	private Map<String, List<Node>> piecesByNom = new HashMap<String, List<Node>>();
	private Map<String, List<Appearance>> piecesAppearenceByNom = new HashMap<String, List<Appearance>>();
	
	/**
	 * Creer une vue sur une frame awt
	 * 
	 * @param frame
	 *            La frame awt dans laquelle va se dessiner la vue
	 */
	public Java3DView(Frame frame) {
		// Le canvas 3D
		Canvas3D canvas3D = new Canvas3D(SimpleUniverse
				.getPreferredConfiguration());
		frame.add(BorderLayout.CENTER, canvas3D);

		// L'univers
		universe = new SimpleUniverse(canvas3D);
		universe.getViewingPlatform().setNominalViewingTransform();
		
		// La scne principale
		rootObject = new BranchGroup();

		// Le transforme groupe principal
		rootTransform = new TransformGroup();
		rootTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		rootTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		rootTransform.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		rootTransform.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		rootTransform.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);

		Transform3D transform = new Transform3D();
		rootTransform.setTransform(transform);
		rootObject.addChild(rootTransform);

		// Ajout le support de la souris
		addSourisSupport();
		
		// Ajout les axes
		//createAxis();

		// Petit fond gris
		Background background = new Background(0.94f, 0.94f, 0.94f);
		background.setApplicationBounds(new BoundingSphere());
		rootObject.addChild(background);

		// On ajoute la scne
		universe.addBranchGraph(rootObject);
	}
	
	/**
	 * 
	 
	private void createAxis() {
		Appearance appearance = new Appearance();
		ColoringAttributes coloringAttributes = new ColoringAttributes();
		coloringAttributes.setColor(new Color3f(0,0,0));
		appearance.setColoringAttributes(coloringAttributes);
		
		LineArray axisX = new LineArray(2,LineArray.COORDINATES|LineArray.COLOR_3);
		axisX.setCoordinate(0,new Point3f((float)-ISomaEngine.BORNE_MIN * ECHELLE,0f,0f));
		axisX.setCoordinate(1,new Point3f((float)ISomaEngine.BORNE_MAX * ECHELLE,0f,0f));
		rootTransform.addChild(new Shape3D(axisX, appearance));
		
		LineArray axisY = new LineArray(2,LineArray.COORDINATES|LineArray.COLOR_3);
		axisY.setCoordinate(0,new Point3f(0f,(float)-ISomaEngine.BORNE_MIN * ECHELLE,0f));
		axisY.setCoordinate(1,new Point3f(0f,(float)ISomaEngine.BORNE_MAX * ECHELLE,0f));
		rootTransform.addChild(new Shape3D(axisY, appearance));
		
		LineArray axisZ = new LineArray(2,LineArray.COORDINATES|LineArray.COLOR_3);
		axisZ.setCoordinate(0,new Point3f(0f,0f,(float)-ISomaEngine.BORNE_MIN * ECHELLE));
		axisZ.setCoordinate(1,new Point3f(0f,0f,(float)ISomaEngine.BORNE_MAX * ECHELLE));
		rootTransform.addChild(new Shape3D(axisZ, appearance));
	}*/

	/**
	 * Ajoute le support de la souris
	 */
	private void addSourisSupport() {
		MouseRotate myMouseRotate = new MouseRotate();
		myMouseRotate.setTransformGroup(rootTransform);
		myMouseRotate.setSchedulingBounds(sphere);
		rootObject.addChild(myMouseRotate);

		MouseTranslate myMouseTranslate = new MouseTranslate();
		myMouseTranslate.setTransformGroup(rootTransform);
		myMouseTranslate.setSchedulingBounds(sphere);
		rootObject.addChild(myMouseTranslate);

		MouseZoom myMouseZoom = new MouseZoom();
		myMouseZoom.setTransformGroup(rootTransform);
		myMouseZoom.setSchedulingBounds(sphere);
		rootObject.addChild(myMouseZoom);
	}

	/**
	 * Dessine les pices dans l'univers 3D
	 * 
	 * @param piece
	 *            La pice  dessiner
	 * @param visible 
	 */
	private void drawPiece(Piece piece, boolean visible) {
		piecesByNom.put(piece.getNom(), new ArrayList<Node>());
		piecesAppearenceByNom.put(piece.getNom(), new ArrayList<Appearance>());
		Color3f color3f = colorsByPiece.get(piece.getNom());
		if( color3f == null )
			color3f = new Color3f((float) Math.abs(Math.max(-0.6, Math.min(0.6, Math.random()))),
				(float) Math.abs(Math.max(-0.6, Math.min(0.6, Math.random()))), (float) Math.abs(Math.max(-0.6, Math.min(0.6, Math.random()))));

		BranchGroup pieceBranchGroup = new BranchGroup();

		double decalage = 0;

		for (int i = 0; i < piece.getCubes().length; i++) {
			Cube cube = piece.getCubes()[i];

			TransformGroup cubeTransformGroup = new TransformGroup();

			Transform3D transform3d = new Transform3D();
			Vector3d v = new Vector3d(decalage + cube.getX()
					* ECHELLE, decalage + cube.getY() * ECHELLE, decalage
					+ cube.getZ() * ECHELLE);
			transform3d.setTranslation(v);

			cubeTransformGroup.setTransform(transform3d);
			
			Appearance appearance = new Appearance();
			Node node = cube2Node(cube, color3f, appearance, visible);
			node.setUserData(cube);
			piecesByNom.get(piece.getNom()).add(node);
			piecesAppearenceByNom.get(piece.getNom()).add(appearance);

			cubeTransformGroup.addChild(node);

			pieceBranchGroup.addChild(cubeTransformGroup);
		}

		// Ajoute la pice
		branchGroup.addChild(pieceBranchGroup);
	}

	/**
	 * Dessine les pices dans l'univers 3D
	 * 
	 * @param pieces
	 *            Les pices  dessiner
	 */
	public void drawPieces(Piece[] pieces, boolean visible) {
		createScene();
		piecesByNom.clear();
		piecesAppearenceByNom.clear();
		for (int i = 0; i < pieces.length; i++) {
			drawPiece(pieces[i], visible);
		}
		branchGroup.compile();
		rootTransform.addChild(branchGroup);
	}
	
	/**
	 * Montre la pice
	 */
	public void setPieceVisible(String nomPiece, boolean visible)
	{
		if( piecesByNom.get(nomPiece) != null )
		{
			for(Appearance appearance : piecesAppearenceByNom.get(nomPiece))
			{
				appearance.getRenderingAttributes().setVisible(visible);
			}
		}
	}

	/**
	 * 
	 * @param cube
	 * @return
	 */
	private Node cube2Node(Cube cube, Color3f color3f, Appearance appearance, boolean visible) {
		BranchGroup group = new BranchGroup();

		ColoringAttributes coloringAttributes = new ColoringAttributes();
		coloringAttributes.setColor(color3f);
		appearance.setColoringAttributes(coloringAttributes);
		
		RenderingAttributes renderingAttributes = new RenderingAttributes();
		renderingAttributes.setCapability(RenderingAttributes.ALLOW_VISIBLE_READ);
		renderingAttributes.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);
		renderingAttributes.setVisible(visible);
		appearance.setRenderingAttributes(renderingAttributes);

		float interval = ECHELLE / 2;
		
		coloringAttributes = new ColoringAttributes();
		coloringAttributes.setColor(color3f);
		appearance.setColoringAttributes(coloringAttributes);

		Box box = new Box(interval, interval, interval, appearance);
		group.addChild(box);
		
		return group;
	}

	/**
	 * 
	 */
	private void createScene() {
		if (branchGroup != null) {
			branchGroup.detach();
			rootTransform.removeChild(branchGroup);
		}

		branchGroup = new BranchGroup();
		branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		branchGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		branchGroup.setCapability(BranchGroup.ALLOW_DETACH);
	}

	/**
	 * 
	 */
	public void clear() {
		piecesAppearenceByNom.clear();
		piecesByNom.clear();
		createScene();
		rootTransform.addChild(branchGroup);
	}
}
